@@ -37,7 +37,7 @@
        Keith Owens <kaos@ocs.com.au> April 2000.
    */
 
-#ident "$Id: depmod.c 1.26 Sat, 21 Oct 2000 15:17:56 +1100 kaos $"
+#ident "$Id: depmod.c 1.31.1.3 Thu, 16 Nov 2000 11:40:09 +1100 kaos $"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -81,20 +81,20 @@ struct pci_device_id {
 	unsigned int vendor, device;		/* Vendor and device ID or PCI_ANY_ID */
 	unsigned int subvendor, subdevice;	/* Subsystem ID's or PCI_ANY_ID */
 	unsigned int class, class_mask;		/* (class,subclass,prog-if) triplet */
-	unsigned long driver_data;		/* Data private to the driver */
+	unsigned tgt_long driver_data;		/* Data private to the driver */
 };
 
 /* Extracted from 2.4.0-test10-pre1 /include/linux/isapnp.h */
 struct isapnp_device_id {
 	unsigned short card_vendor, card_device;
 	unsigned short vendor, function;
-	unsigned long driver_data;	/* data private to the driver */
+	unsigned tgt_long driver_data;		/* data private to the driver */
 };
 
 /* Extracted from 2.4.0-test10-pre1 /include/linux/isapnp.h */
 #define ISAPNP_CARD_DEVS	8
 struct isapnp_card_id {
-	unsigned long driver_data;	/* data private to the driver */
+	unsigned tgt_long driver_data;	/* data private to the driver */
 	unsigned short card_vendor, card_device;
 	struct {
 		unsigned short vendor, function;
@@ -149,7 +149,7 @@ struct usb_device_id {
 	/*
 	 * for driver's use; not involved in driver matching.
 	 */
-	unsigned long	driver_info;
+	unsigned tgt_long		driver_info;
 };
 
 typedef struct MODULE {
@@ -363,7 +363,7 @@ static void extract_pci_device_id(struct obj_file *f, void *image, unsigned long
 	struct pci_device_id pci_device;
 	ElfW(Addr) ref_pci;
 	ElfW(Addr) ref_ref_pci;
-	unsigned long pci_device_size;
+	unsigned tgt_long pci_device_size;
 	ref_pci = obj_symbol_final_value(f, obj_find_symbol(f, "__module_pci_device_size"));
 	/* Kernels before 2.4.0-test10-pre1 do not have __module_pci_device_size */
 	if (ref_pci) {
@@ -374,7 +374,8 @@ static void extract_pci_device_id(struct obj_file *f, void *image, unsigned long
 	else
 		pci_device_size = sizeof(pci_device);
 	if (pci_device_size != sizeof(pci_device)) {
-		error("Unexpected value (%ld) for pci_device_size%s", pci_device_size, has_kernel_changed);
+		error("Unexpected value (%" tgt_long_fmt "d) in '%s' for pci_device_size%s",
+			pci_device_size, f->filename, has_kernel_changed);
 		exit(-1);
 	}
 	ref_ref_pci = obj_symbol_final_value(f, obj_find_symbol(f, "__module_pci_device_table"));
@@ -399,13 +400,14 @@ static void extract_isapnp_device_id(struct obj_file *f, void *image, unsigned l
 {
 	struct isapnp_device_id isapnp_device;
 	ElfW(Addr) ref_isapnp, ref_ref_isapnp;
-	unsigned long isapnp_device_size;
+	unsigned tgt_long isapnp_device_size;
 	ref_isapnp = obj_symbol_final_value(f, obj_find_symbol(f, "__module_isapnp_device_size"));
 	if (!in_range(f, m_size, ref_isapnp, sizeof(isapnp_device_size)))
 		return;
 	memcpy(&isapnp_device_size, (char *)image + ref_isapnp - f->baseaddr, sizeof(isapnp_device_size));
 	if (isapnp_device_size != sizeof(isapnp_device)) {
-		error("Unexpected value (%ld) for isapnp_device_size%s", isapnp_device_size, has_kernel_changed);
+		error("Unexpected value (%" tgt_long_fmt "d) in '%s' for isapnp_device_size%s",
+			isapnp_device_size, f->filename, has_kernel_changed);
 		exit(-1);
 	}
 	ref_ref_isapnp = obj_symbol_final_value(f, obj_find_symbol(f, "__module_isapnp_device_table"));
@@ -430,13 +432,14 @@ static void extract_isapnp_card_id(struct obj_file *f, void *image, unsigned lon
 {
 	struct isapnp_card_id isapnp_card;
 	ElfW(Addr) ref_isapnp, ref_ref_isapnp;
-	unsigned long isapnp_card_size;
+	unsigned tgt_long isapnp_card_size;
 	ref_isapnp = obj_symbol_final_value(f, obj_find_symbol(f, "__module_isapnp_card_size"));
 	if (!in_range(f, m_size, ref_isapnp, sizeof(isapnp_card_size)))
 		return;
 	memcpy(&isapnp_card_size, (char *)image + ref_isapnp - f->baseaddr, sizeof(isapnp_card_size));
 	if (isapnp_card_size != sizeof(isapnp_card)) {
-		error("Unexpected value (%ld) for isapnp_card_size%s", isapnp_card_size, has_kernel_changed);
+		error("Unexpected value (%" tgt_long_fmt "d) in '%s' for isapnp_card_size%s",
+			isapnp_card_size, f->filename, has_kernel_changed);
 		exit(-1);
 	}
 	ref_ref_isapnp = obj_symbol_final_value(f, obj_find_symbol(f, "__module_isapnp_card_table"));
@@ -461,13 +464,14 @@ static void extract_usb_device_id(struct obj_file *f, void *image, unsigned long
 {
 	struct usb_device_id usb_device;
 	ElfW(Addr) ref_usb, ref_ref_usb;
-	unsigned long usb_device_size;
+	unsigned tgt_long usb_device_size;
 	ref_usb = obj_symbol_final_value(f, obj_find_symbol(f, "__module_usb_device_size"));
 	if (!in_range(f, m_size, ref_usb, sizeof(usb_device_size)))
 		return;
 	memcpy(&usb_device_size, (char *)image + ref_usb - f->baseaddr, sizeof(usb_device_size));
 	if (usb_device_size != sizeof(usb_device)) {
-		error("Unexpected value (%ld) for usb_device_size%s", usb_device_size, has_kernel_changed);
+		error("Unexpected value (%" tgt_long_fmt "d) in '%s' for usb_device_size%s",
+			usb_device_size, f->filename, has_kernel_changed);
 		exit(-1);
 	}
 	ref_ref_usb = obj_symbol_final_value(f, obj_find_symbol(f, "__module_usb_device_table"));
@@ -477,7 +481,7 @@ static void extract_usb_device_id(struct obj_file *f, void *image, unsigned long
 	while (in_range(f, m_size, ref_usb, sizeof(usb_device))) {
 		memcpy(&usb_device, (char *)image + ref_usb - f->baseaddr, sizeof(usb_device));
 		ref_usb += sizeof(usb_device);
-		if (!usb_device.idVendor)
+		if (!usb_device.idVendor && !usb_device.bDeviceClass && !usb_device.bInterfaceClass && !usb_device.driver_info)
 			break;
 		mod->usb_device = xrealloc(mod->usb_device, ++(mod->n_usb_device)*sizeof(*(mod->usb_device)));
 		mod->usb_device[mod->n_usb_device-1] = usb_device;
@@ -759,9 +763,23 @@ static int addksyms(char *file_syms)
 				if (*p != '?')
 					continue;
 				p = strtok(NULL, " \t\n");
-				if (strncmp(p, "__kstrtab_", 10))
+				if (!strncmp(p, "__kstrtab_", 10))
+					p += 10;
+				else if (!strncmp(p, "__export_priv_", 14)) {
+					/* Sparc has some weird exported symbols marked
+					 * __export_priv_ instead of the normal __kstrtab_.
+					 * Replace the 'v' with '_' and point at the start of
+					 * the '__' before the name.  I see no good reason
+					 * to use __export_priv_, but for compatibility with
+					 * old sparc kernels, it is supported.  Try to remove
+					 * __export_priv_ from sparc and remove this code
+					 * four releases after the last kernel that uses
+					 * __export_priv_.
+					 */
+					p += 12;
+					*p = '_';
+				} else
 					continue;
-				p += 10;
 			}
 			assert(n_syms < MAX_MAP_SYM);
 			symtab[n_syms++] = addsym(p, mod, SYM_DEFINED, 0);
@@ -785,18 +803,51 @@ static int addksyms(char *file_syms)
 	return 0;
 }
 
+/* Open a generated file for output */
+static FILE *gen_file_open(const struct gen_files *gf)
+{
+	FILE *f = fopen(gf->name, "w");
+	if (!f) {
+		error("Can't open %s for writing", gf->name);
+		exit(-1);
+	}
+	return(f);
+}
+
+static
+char *shortname(const char *name)
+{
+	char *name2 = xstrdup(name);
+	char *s = strrchr(name2, '/');
+	int l;
+	if (s) {
+		++s;
+		l = strlen(s);
+		if (l > 3 && strcmp(s+l-3, ".gz") == 0) {
+			*(s+l-3) = '\0';
+			l -= 3;
+		}
+		if (l > 2 && strcmp(s+l-2, ".o") == 0) {
+			*(s+l-2) = '\0';
+			l -= 2;
+		} else if (l > 4 && strcmp(s+l-4, ".mod") == 0) {
+			*(s+l-4) = '\0';
+			l -= 4;
+		}
+	}
+	else
+		s = name2;	/* No '/' in name, strange */
+	return(s);
+}
+
 /*
  *	Format the dependancy list of a module into a simple makefile.
- *	Print the dependancies in the depfile (or stdout if depfile is NULL).
- *	Print the pcimap in the pcimapfile (or stdout if pcimapfile is NULL).
- *	Print the isapnpmap in the isapnpmapfile (or stdout if isapnpmapfile is NULL).
- *	Print the usbmap in the usbmapfile (or stdout if usbmapfile is NULL).
+ *	Print the dependancies in the depfile (or stdout if nflag is true).
+ *	Print the pcimap in the pcimapfile (or stdout if nflag is true).
+ *	Print the isapnpmap in the isapnpmapfile (or stdout if nflag is true).
+ *	Print the usbmap in the usbmapfile (or stdout if nflag is true).
  */
-static void prtdepend(char *base_dir,
-		      const char *pdepfile,
-		      const char *ppcimapfile,
-		      const char *pisapnpmapfile,
-		      const char *pusbmapfile)
+static void prtdepend(char *base_dir, int nflag)
 {
 	FILE *dep = stdout;
 	FILE *pcimap = stdout;
@@ -807,32 +858,11 @@ static void prtdepend(char *base_dir,
 	int i;
 	int skipchars;	/* For depmod -a in image of a tree */
 
-	if (pdepfile != NULL) {
-		if ((dep = fopen(pdepfile, "w")) == NULL) {
-			error("Can't open %s for writing", pdepfile);
-			exit(-1);
-		}
-	}
-
-	if (ppcimapfile != NULL) {
-		if ((pcimap = fopen(ppcimapfile, "w")) == NULL) {
-			error("Can't open %s for writing", ppcimapfile);
-			exit(-1);
-		}
-	}
-
-	if (pisapnpmapfile != NULL) {
-		if ((isapnpmap = fopen(pisapnpmapfile, "w")) == NULL) {
-			error("Can't open %s for writing", pisapnpmapfile);
-			exit(-1);
-		}
-	}
-
-	if (pusbmapfile != NULL) {
-		if ((usbmap = fopen(pusbmapfile, "w")) == NULL) {
-			error("Can't open %s for writing", pusbmapfile);
-			exit(-1);
-		}
+	if (!nflag) {
+		dep = gen_file_open(gen_file+GEN_DEPFILE);
+		pcimap = gen_file_open(gen_file+GEN_PCIMAPFILE);
+		isapnpmap = gen_file_open(gen_file+GEN_ISAPNPMAPFILE);
+		usbmap = gen_file_open(gen_file+GEN_USBMAPFILE);
 	}
 
 	skipchars = strlen(base_dir);
@@ -889,35 +919,15 @@ static void prtdepend(char *base_dir,
 	}
 
 	ptmod = modules;
-	fprintf(pcimap, "# module             vendor     device     subvendor  subdevice  class      class_mask driver_data\n");
+	fprintf(pcimap, "# pci module         vendor     device     subvendor  subdevice  class      class_mask driver_data\n");
 	for (i = 0; i < n_modules; i++, ptmod++) {
-		int j, l;
+		int j;
 		struct pci_device_id *pci_device = ptmod->pci_device;
-		char *name, *shortname;
 		if (!ptmod->n_pci_device)
 			continue;
-		name = xstrdup(ptmod->name);
-		shortname = strrchr(name, '/');
-		if (!shortname)
-			shortname = name;	/* should never happen */
-		else {
-			++shortname;
-			l = strlen(shortname);
-			if (l > 3 && strcmp(shortname+l-3, ".gz") == 0) {
-				*(shortname+l-3) = '\0';
-				l -= 3;
-			}
-			if (l > 2 && strcmp(shortname+l-2, ".o") == 0) {
-				*(shortname+l-2) = '\0';
-				l -= 2;
-			} else if (l > 4 && strcmp(shortname+l-4, ".mod") == 0) {
-				*(shortname+l-4) = '\0';
-				l -= 4;
-			}
-		}
 		for (j = 0; j < ptmod->n_pci_device; j++, pci_device++) {
-			fprintf(pcimap, "%-20s 0x%0*x 0x%0*x 0x%0*x 0x%0*x 0x%0*x 0x%0*x 0x%0*lx\n",
-				shortname,
+			fprintf(pcimap, "%-20s 0x%0*x 0x%0*x 0x%0*x 0x%0*x 0x%0*x 0x%0*x 0x%0*" tgt_long_fmt "x\n",
+				shortname(ptmod->name),
 				2*sizeof(pci_device->vendor), pci_device->vendor,
 				2*sizeof(pci_device->device), pci_device->device,
 				2*sizeof(pci_device->subvendor), pci_device->subvendor,
@@ -926,40 +936,19 @@ static void prtdepend(char *base_dir,
 				2*sizeof(pci_device->class_mask), pci_device->class_mask,
 				2*sizeof(pci_device->driver_data), pci_device->driver_data);
 		}
-		free(name);
 	}
 
 	ptmod = modules;
-	fprintf(isapnpmap, "# module             cardvendor carddevice driver_data vendor     function   ...\n");
+	fprintf(isapnpmap, "# isapnp module      cardvendor carddevice driver_data vendor     function   ...\n");
 	for (i = 0; i < n_modules; i++, ptmod++) {
 		int j, l;
 		struct isapnp_device_id *isapnp_device = ptmod->isapnp_device;
 		struct isapnp_card_id *isapnp_card = ptmod->isapnp_card;
-		char *name, *shortname;
 		if (!ptmod->n_isapnp_device && !ptmod->n_isapnp_card)
 			continue;
-		name = xstrdup(ptmod->name);
-		shortname = strrchr(name, '/');
-		if (!shortname)
-			shortname = name;	/* should never happen */
-		else {
-			++shortname;
-			l = strlen(shortname);
-			if (l > 3 && strcmp(shortname+l-3, ".gz") == 0) {
-				*(shortname+l-3) = '\0';
-				l -= 3;
-			}
-			if (l > 2 && strcmp(shortname+l-2, ".o") == 0) {
-				*(shortname+l-2) = '\0';
-				l -= 2;
-			} else if (l > 4 && strcmp(shortname+l-4, ".mod") == 0) {
-				*(shortname+l-4) = '\0';
-				l -= 4;
-			}
-		}
 		for (j = 0; j < ptmod->n_isapnp_device; j++, isapnp_device++) {
-			fprintf(isapnpmap, "%-20s 0x%0*x     0x%0*x     0x%0*lx  0x%0*x     0x%0*x\n",
-				shortname,
+			fprintf(isapnpmap, "%-20s 0x%0*x     0x%0*x     0x%0*" tgt_long_fmt "x  0x%0*x     0x%0*x\n",
+				shortname(ptmod->name),
 				2*sizeof(isapnp_device->card_vendor), isapnp_device->card_vendor,
 				2*sizeof(isapnp_device->card_device), isapnp_device->card_device,
 				2*sizeof(isapnp_device->driver_data), isapnp_device->driver_data,
@@ -967,8 +956,8 @@ static void prtdepend(char *base_dir,
 				2*sizeof(isapnp_device->function), isapnp_device->function);
 		}
 		for (j = 0; j < ptmod->n_isapnp_card; j++, isapnp_card++) {
-			fprintf(isapnpmap, "%-20s 0x%0*x     0x%0*x     0x%0*lx ",
-				shortname,
+			fprintf(isapnpmap, "%-20s 0x%0*x     0x%0*x     0x%0*" tgt_long_fmt "x ",
+				shortname(ptmod->name),
 				2*sizeof(isapnp_card->card_vendor), isapnp_card->card_vendor,
 				2*sizeof(isapnp_card->card_device), isapnp_card->card_device,
 				2*sizeof(isapnp_card->driver_data), isapnp_card->driver_data);
@@ -981,43 +970,22 @@ static void prtdepend(char *base_dir,
 			}
 			fprintf(isapnpmap, "\n");
 		}
-		free(name);
 	}
 
 	ptmod = modules;
-	fprintf(usbmap, "# module             idVendor idProduct bcdDevice_lo bcdDevice_hi"
+	fprintf(usbmap, "# usb module         idVendor idProduct bcdDevice_lo bcdDevice_hi"
 		" bDeviceClass bDeviceSubClass bDeviceProtocol bInterfaceClass bInterfaceSubClass"
 		" bInterfaceProtocol driver_info\n");
 	for (i = 0; i < n_modules; i++, ptmod++) {
-		int j, l;
+		int j;
 		struct usb_device_id *usb_device = ptmod->usb_device;
-		char *name, *shortname;
 		if (!ptmod->n_usb_device)
 			continue;
-		name = xstrdup(ptmod->name);
-		shortname = strrchr(name, '/');
-		if (!shortname)
-			shortname = name;	/* should never happen */
-		else {
-			++shortname;
-			l = strlen(shortname);
-			if (l > 3 && strcmp(shortname+l-3, ".gz") == 0) {
-				*(shortname+l-3) = '\0';
-				l -= 3;
-			}
-			if (l > 2 && strcmp(shortname+l-2, ".o") == 0) {
-				*(shortname+l-2) = '\0';
-				l -= 2;
-			} else if (l > 4 && strcmp(shortname+l-4, ".mod") == 0) {
-				*(shortname+l-4) = '\0';
-				l -= 4;
-			}
-		}
 		for (j = 0; j < ptmod->n_usb_device; j++, usb_device++) {
 			fprintf(usbmap, "%-20s 0x%0*x   0x%0*x    0x%0*x       0x%0*x       "
 				"0x%0*x         0x%0*x            0x%0*x            0x%0*x            0x%0*x            "
-				"0x%0*x            0x%0*lx\n",
-				shortname,
+				"0x%0*x            0x%0*" tgt_long_fmt "x\n",
+				shortname(ptmod->name),
 				2*sizeof(usb_device->idVendor), usb_device->idVendor,
 				2*sizeof(usb_device->idProduct), usb_device->idProduct,
 				2*sizeof(usb_device->bcdDevice_lo), usb_device->bcdDevice_lo,
@@ -1030,17 +998,16 @@ static void prtdepend(char *base_dir,
 				2*sizeof(usb_device->bInterfaceProtocol), usb_device->bInterfaceProtocol,
 				2*sizeof(usb_device->driver_info), usb_device->driver_info);
 		}
-		free(name);
 	}
 
-	/* Close depfile last, critical timestamp */
-	if (ppcimapfile != NULL)
+	if (pcimap != stdout)
 		fclose(pcimap);
-	if (pisapnpmapfile != NULL)
+	if (isapnpmap != stdout)
 		fclose(isapnpmap);
-	if (pusbmapfile != NULL)
+	if (usbmap != stdout)
 		fclose(usbmap);
-	if (pdepfile != NULL)
+	/* Close depfile last, it's timestamp is critical */
+	if (dep != stdout)
 		fclose(dep);
 }
 
@@ -1217,7 +1184,7 @@ int DEPMOD_MAIN(int argc, char **argv)
 		/* option -a is the default without arguments */
 		if (argc > 0) {
 			if (config_read(0, *argv, base_dir, conf_file) == -1) {
-				error("%s does not exist", depfile);
+				error("%s does not exist", gen_file[GEN_DEPFILE].name);
 				return -1;
 			}
 		} else if (config_read(0, NULL, base_dir, conf_file) == -1) {
@@ -1230,11 +1197,7 @@ int DEPMOD_MAIN(int argc, char **argv)
 			for (i = 0; g && i < g->pathc; i++)
 				loadobj(g->pathv[i], ignore_suffix);
 			resolve();
-			prtdepend(base_dir,
-				  nflag ? NULL : depfile,
-				  nflag ? NULL : pcimapfile,
-				  nflag ? NULL : isapnpmapfile,
-				  nflag ? NULL : usbmapfile);
+			prtdepend(base_dir, nflag);
 		}
 	} else {
 		/* not stdmode */
@@ -1242,7 +1205,7 @@ int DEPMOD_MAIN(int argc, char **argv)
 			for (; argc > 0 && ret != -1; ++argv, --argc)
 				loadobj(*argv, ignore_suffix);
 			resolve();
-			prtdepend(base_dir, NULL, NULL, NULL, NULL);
+			prtdepend(base_dir, 1);
 		}
 	}
 
