@@ -63,10 +63,6 @@ extern int nmodpath;
 extern struct EXEC_TYPE *execs;
 extern int nexecs;
 extern char *insmod_opt;
-extern char *depfile;
-extern char *pcimapfile;
-extern char *isapnpmapfile;
-extern char *usbmapfile;
 extern char *config_file;
 extern char *optlist[];
 extern char *prune[];
@@ -80,6 +76,23 @@ extern OPT_LIST *aliases;
 extern time_t config_mtime;
 extern int root_check_off;	/* Check modules are owned by root? */
 
+/* Information about generated files */
+struct gen_files {
+	char *base;		/* xxx in /lib/modules/`uname -r`/modules.xxx */
+	char *name;             /* name actually used */
+	time_t mtime;
+};
+
+extern struct gen_files gen_file[];
+extern const int gen_file_count;
+/* The enum order must match the gen_file initialization order in config.c */
+enum gen_file_enum {
+	GEN_PCIMAPFILE,
+	GEN_ISAPNPMAPFILE,
+	GEN_USBMAPFILE,
+	GEN_DEPFILE,
+};
+
 char *fgets_strip(char *buf, int sizebuf, FILE * fin, int *lineno);
 int config_read(int all, char *force_ver, char *base_dir, char *conf_file);
 GLOB_LIST *config_lstmod(const char *match, const char *type, int first_only);
