@@ -23,7 +23,7 @@
 #ifndef MODUTILS_MODULE_H
 #define MODUTILS_MODULE_H 1
 
-#ident "$Id: module.h 1.5.1.3 Thu, 16 Nov 2000 11:40:09 +1100 kaos $"
+#ident "$Id: module.h 1.5.1.4 Wed, 22 Nov 2000 22:05:39 +1100 kaos $"
 
 /* This file contains the structures used by the 2.0 and 2.1 kernels.
    We do not use the kernel headers directly because we do not wish
@@ -207,6 +207,6 @@ int delete_module(const char *);
 /* In safe mode the last parameter is forced to be a module name and meta
  * expansion is not allowed on that name.
  */
-extern unsigned long safemode;
+extern unsigned int safemode;
 
 #endif /* module.h */
