@@ -24,7 +24,7 @@
 #ifndef MODUTILS_OBJ_H
 #define MODUTILS_OBJ_H 1
 
-#ident "$Id: obj.h 1.11 Fri, 08 Sep 2000 16:46:27 +1100 kaos $"
+#ident "$Id: obj.h 1.11.1.1 Tue, 14 Nov 2000 23:08:54 +1100 kaos $"
 
 /* The relocatable object is manipulated using elfin types.  */
 
@@ -42,14 +42,14 @@
 # endif
 #endif
 
-#if defined(__sparc__) && !defined(__sparc_v9__)
-# if ELFCLASSM == ELFCLASS32
+#if defined(COMMON_3264) && defined(ONLY_32)
 #  define ObjW(x)  obj32_ ## x
-# else
-#  define ObjW(x)  obj64_ ## x
-# endif
 #else
-#define ObjW(x)    obj_ ## x
+#  if defined(COMMON_3264) && defined(ONLY_64)
+#    define ObjW(x)  obj64_ ## x
+#  else
+#    define ObjW(x)    obj_ ## x
+#  endif
 #endif
 
 /* For some reason this is missing from lib5.  */
