@@ -23,8 +23,9 @@
 #ifndef MODUTILS_UTIL_H
 #define MODUTILS_UTIL_H 1
 
-#ident "$Id: util.h 1.6 Mon, 07 Aug 2000 01:20:16 +1000 kaos $"
+#ident "$Id: util.h 1.6.1.3 Thu, 16 Nov 2000 09:29:06 +1100 kaos $"
 
+#include <stdio.h>
 #include <sys/stat.h>
 
 #define SHELL_META "&();|<>$`\"'\\!{}[]~=+:?*" /* Sum of bj0rn and Debian */
@@ -32,6 +33,8 @@
 void *xmalloc(size_t);
 void *xrealloc(void *, size_t);
 char *xstrdup(const char *);
+char *xstrcat(char *, const char *, size_t);
+int   xsystem(const char *, char *const[]);
 int   arch64(void);
 
 typedef int (*xftw_func_t)(const char *, const struct stat *);
@@ -66,7 +69,11 @@ typedef struct {
 	int pathc;       /* Count of paths matched so far  */
 	char **pathv;    /* List of matched pathnames.  */
 } GLOB_LIST;
-int meta_expand(char *pt, GLOB_LIST *g, char *base_dir, char *version);
+int meta_expand(char *pt, GLOB_LIST *g, char *base_dir, char *version, int type);
+#define ME_BUILTIN_COMMAND	1
+#define ME_SHELL_COMMAND	2
+#define ME_GLOB			4
+#define ME_ALL			(ME_GLOB|ME_SHELL_COMMAND|ME_BUILTIN_COMMAND)
 
 extern void snap_shot(const char *module_name, int number);
 
@@ -87,4 +94,6 @@ void gzf_close(int fd);
 
 #endif /* CONFIG_USE_ZLIB */
 
+static const char symprefix[] = "__insmod_";
+
 #endif /* util.h */
