@@ -50,9 +50,11 @@
        Keith Owens <kaos@ocs.com.au> August 2000.
      Add insmod -O, move print map before sys_init_module.
        Keith Owens <kaos@ocs.com.au> October 2000.
+     Add insmod -S.
+       Keith Owens <kaos@ocs.com.au> November 2000.
    */
 
-#ident "$Id: insmod.c 1.29 Sat, 21 Oct 2000 15:31:41 +1100 kaos $"
+#ident "$Id: insmod.c 1.30.1.2 Tue, 14 Nov 2000 23:36:27 +1100 kaos $"
 
 #include <sys/types.h>
 #include <stdlib.h>
@@ -81,7 +83,7 @@
 /*======================================================================*/
 
 static int flag_force_load = 0;
-static int flag_silent_poll = 0;
+static int flag_silent_probe = 0;
 static int flag_export = 1;
 static int flag_load_map = 0;
 static int flag_ksymoops = 1;
@@ -137,9 +139,11 @@ static int ncv_plen = 0;
 static void set_ncv_prefix(char *prefix)
 {
 	static char derived_prefix[256];
-	static const char well_known_symbol[] = "get_module_symbol_R";
+	static const char *well_known_symbol[] = { "get_module_symbol_R",
+						   "inter_module_get_R",
+						 };
 	struct module_symbol *s;
-	int i, l, pl;
+	int i, j, l, m, pl;
 	const char *name;
 	char *p;
 
@@ -149,18 +153,22 @@ static void set_ncv_prefix(char *prefix)
 	if (prefix)
 		ncv_prefix = prefix;
 	else {
-		/* Extract the prefix (if any) from the well known symbol */
+		/* Extract the prefix (if any) from well known symbols */
 		for (i = 0, s = ksyms; i < nksyms; ++i, ++s) {
 			name = (char *) s->name;
-			if (strncmp(name, well_known_symbol, sizeof(well_known_symbol)-1) == 0) {
-				l = strlen(name);
-				pl = l - sizeof(well_known_symbol) - 7;
-				if (pl < 0 || pl > sizeof(derived_prefix)-1)
+			l = strlen(name);
+			for (j = 0; j < sizeof(well_known_symbol)/sizeof(well_known_symbol[0]); ++j) {
+				m = strlen(well_known_symbol[j]);
+				if (m + 8 > l ||
+				    strncmp(name, well_known_symbol[j], m))
+					continue;
+				pl = l - m - 8;
+				if (pl > sizeof(derived_prefix)-1)
 					continue;	/* Prefix is wrong length */
 				/* Must end with 8 hex digits */
 				(void) strtoul(name+l-8, &p, 16);
 				if (*p == 0) {
-					strncpy(derived_prefix, name+sizeof(well_known_symbol)-1, pl);
+					strncpy(derived_prefix, name+m, pl);
 					*(derived_prefix+pl) = '\0';
 					ncv_prefix = derived_prefix;
 					break;
@@ -177,7 +185,7 @@ static void set_ncv_prefix(char *prefix)
 	}
 	ncv_plen = strlen(ncv_prefix);
 	if (flag_verbose)
-		printf("Symbol version prefix '%s'\n", ncv_prefix);
+		lprintf("Symbol version prefix '%s'", ncv_prefix);
 }
 
 static int ncv_strcmp(const char *a, const char *b)
@@ -617,7 +625,6 @@ static void add_ksymoops_symbols(struct obj_file *f, const char *filename,
 	struct obj_symbol *sym;
 	char *name, *absolute_filename;
 	char str[STRVERSIONLEN], real[PATH_MAX];
-	const char symprefix[] = "__insmod_";
 	int i, l, lm_name, lfilename, use_ksymtab, version;
 	struct stat statbuf;
 
@@ -871,13 +878,13 @@ static int process_module_arguments(struct obj_file *f, int argc, char **argv)
 					charssize = strtoul(fmt + 1, (char **) NULL, 10);
 
 					/* Check length */
-					if (strlen(str) >= charssize) {
+					if (strlen(str) >= charssize-1) {
 						error("string too long for %s (max %ld)",
 						      key, charssize - 1);
 						return 0;
 					}
 					/* Copy to location */
-					strcpy((char *) loc, str);
+					strcpy((char *) loc, str);	/* safe, see check above */
 					loc += charssize;
 				}
 				/*
@@ -945,7 +952,7 @@ static int process_module_arguments(struct obj_file *f, int argc, char **argv)
 
 /* Add a kallsyms section if the kernel supports all symbols. */
 static int add_kallsyms(struct obj_file *f,
-			struct obj_section **module_kallsyms)
+			struct obj_section **module_kallsyms, int force_kallsyms)
 {
 	struct module_symbol *s;
 	struct obj_file *f_kallsyms;
@@ -968,7 +975,7 @@ static int add_kallsyms(struct obj_file *f,
 			stop = s->value;
 	}
 
-	if (start >= stop)
+	if (start >= stop && !force_kallsyms)
 		return(0);
 
 	/* The kernel contains all symbols, do the same for this module. */
@@ -1238,28 +1245,28 @@ extern void insmod_usage(void);		/* Use the copy in the 64 bit version */
 void insmod_usage(void)
 {
 	fputs("Usage:\n"
-	      "insmod [-fhkmopqsvVxXyY] [-o name] [-P prefix] module [[sym=value]...]\n"
-
+	      "insmod [-fhkLmnpqrsSvVxXyY] [-o module_name] [-O blob_name] [-P prefix] module [ symbol=value ... ]\n"
 	      "\n"
-	      "  module                Filename of a loadable kernel module (*.o)\n"
+	      "  module                Name of a loadable kernel module ('.o' can be omitted)\n"
 	      "  -f, --force           Force loading under wrong kernel version\n"
-	      "  -h, --help            Print this message.\n"
+	      "  -h, --help            Print this message\n"
 	      "  -k, --autoclean       Make module autoclean-able\n"
+	      "  -L, --lock            Prevent simultaneous loads of the same module\n"
 	      "  -m, --map             Generate load map (so crashes can be traced)\n"
 	      "  -n, --noload          Don't load, just show\n"
-	      "  -o NAME, --name=NAME  Set internal module name to NAME\n"
-	      "  -O NAME, --blob=NAME  Save the object as a binary blob in NAME\n"
-	      "  -p, --poll            Poll mode; check if the module matches the kernel\n"
+	      "  -p, --probe           Probe mode; check if the module matches the kernel\n"
 	      "  -q, --quiet           Don't print unresolved symbols\n"
+	      "  -r, --root            Allow root to load modules not owned by root\n"
 	      "  -s, --syslog          Report errors via syslog\n"
+	      "  -S, --kallsyms        Force kallsyms on module\n"
 	      "  -v, --verbose         Verbose output\n"
-	      "  -L, --lock            Prevent simultaneous loads of the same module\n"
 	      "  -V, --version         Show version\n"
 	      "  -x, --noexport        Do not export externs\n"
 	      "  -X, --export          Do export externs (default)\n"
 	      "  -y, --noksymoops      Do not add ksymoops symbols\n"
 	      "  -Y, --ksymoops        Do add ksymoops symbols (default)\n"
-	      "  -r, --root            Allow root to load modules not owned by root\n"
+	      "  -o NAME, --name=NAME  Set internal module name to NAME\n"
+	      "  -O NAME, --blob=NAME  Save the object as a binary blob in NAME\n"
 	      "  -P PREFIX\n"
 	      "      --prefix=PREFIX   Prefix for kernel or module symbols\n"
 	      ,stderr);
@@ -1282,24 +1289,26 @@ int INSMOD_MAIN(int argc, char **argv)
 	char k_strversion[STRVERSIONLEN];
 	struct option long_opts[] = {
 		{"force", 0, 0, 'f'},
+		{"help", 0, 0, 'h'},
 		{"autoclean", 0, 0, 'k'},
+		{"lock", 0, 0, 'L'},
 		{"map", 0, 0, 'm'},
 		{"noload", 0, 0, 'n'},
-		{"name", 1, 0, 'o'},
-		{"blob", 1, 0, 'O'},
-		{"poll", 0, 0, 'p'},
+		{"probe", 0, 0, 'p'},
+		{"poll", 0, 0, 'p'},	/* poll is deprecated, remove in 2.5 */
+		{"quiet", 0, 0, 'q'},
+		{"root", 0, 0, 'r'},
 		{"syslog", 0, 0, 's'},
+		{"kallsyms", 0, 0, 'S'},
 		{"verbose", 0, 0, 'v'},
 		{"version", 0, 0, 'V'},
-		{"lock", 0, 0, 'L'},
-		{"prefix", 1, 0, 'P'},
 		{"noexport", 0, 0, 'x'},
 		{"export", 0, 0, 'X'},
-		{"quiet", 0, 0, 'q'},
 		{"noksymoops", 0, 0, 'y'},
 		{"ksymoops", 0, 0, 'Y'},
-		{"root", 0, 0, 'r'},
-		{"help", 0, 0, 'h'},
+		{"name", 1, 0, 'o'},
+		{"blob", 1, 0, 'O'},
+		{"prefix", 1, 0, 'P'},
 		{0, 0, 0, 0}
 	};
 	char *m_name = NULL;
@@ -1318,6 +1327,7 @@ int INSMOD_MAIN(int argc, char **argv)
 	int dolock = 1; /*Note: was: 0; */
 	int quiet = 0;
 	int exit_status = 1;
+	int force_kallsyms = 0;
 	int i;
 
 	error_file = "insmod";
@@ -1326,12 +1336,15 @@ int INSMOD_MAIN(int argc, char **argv)
 	errors = optind = 0;
 
 	/* Process the command line.  */
-	while ((o = getopt_long(argc, argv, "fhkmno:O:pqsvVxXLP:yYr",
+	while ((o = getopt_long(argc, argv, "fhkLmnpqrsSvVxXyYo:O:P:",
 				&long_opts[0], NULL)) != EOF)
 		switch (o) {
 		case 'f':	/* force loading */
 			flag_force_load = 1;
 			break;
+		case 'h':       /* Print the usage message. */
+			insmod_usage();
+			break;
 		case 'k':	/* module loaded by kerneld, auto-cleanable */
 			flag_autoclean = 1;
 			break;
@@ -1344,21 +1357,21 @@ int INSMOD_MAIN(int argc, char **argv)
 		case 'n':	/* don't load, just check */
 			noload = 1;
 			break;
-		case 'o':	/* name the output module */
-			m_name = optarg;
-			break;
-		case 'O':	/* save the output module object */
-			blob_name = optarg;
-			break;
-		case 'p':	/* silent poll mode */
-			flag_silent_poll = 1;
+		case 'p':	/* silent probe mode */
+			flag_silent_probe = 1;
 			break;
 		case 'q':	/* Don't print unresolved symbols */
 			quiet = 1;
 			break;
+		case 'r':	/* allow root to load non-root modules */
+			root_check_off = !root_check_off;
+			break;
 		case 's':	/* start syslog */
 			setsyslog("insmod");
 			break;
+		case 'S':	/* Force kallsyms */
+			force_kallsyms = 1;
+			break;
 		case 'v':	/* verbose output */
 			flag_verbose = 1;
 			break;
@@ -1377,13 +1390,17 @@ int INSMOD_MAIN(int argc, char **argv)
 		case 'Y':	/* do define ksymoops symbols */
 			flag_ksymoops = 1;
 			break;
-		case 'r':	/* allow root to load non-root modules */
-			root_check_off = !root_check_off;
+
+		case 'o':	/* name the output module */
+			m_name = optarg;
+			break;
+		case 'O':	/* save the output module object */
+			blob_name = optarg;
 			break;
 		case 'P':	/* use prefix on crc */
 			set_ncv_prefix(optarg);
 			break;
-		case 'h':       /* Print the usage message. */
+
 		default:
 			insmod_usage();
 			break;
@@ -1430,9 +1447,9 @@ int INSMOD_MAIN(int argc, char **argv)
 			return 1;
 		}
 		filename = tmp;
-		printf("Using %s\n", filename);
+		lprintf("Using %s", filename);
 	} else if (flag_verbose)
-		printf("Using %s\n", filename);
+		lprintf("Using %s", filename);
 
 	/* And open it.  */
 	if ((fp = gzf_open(filename, O_RDONLY)) == -1) {
@@ -1479,7 +1496,7 @@ int INSMOD_MAIN(int argc, char **argv)
 		if (flag_force_load) {
 			lprintf("Warning: kernel-module version mismatch\n"
 			      "\t%s was compiled for kernel version %s\n"
-				"\twhile this kernel is version %s\n",
+				"\twhile this kernel is version %s",
 				filename, m_strversion, k_strversion);
 		} else {
 			if (!quiet)
@@ -1534,7 +1551,7 @@ int INSMOD_MAIN(int argc, char **argv)
 		goto out;
 
 	/* kallsyms based on relocatable addresses */
-	if (add_kallsyms(f, &kallsyms))
+	if (add_kallsyms(f, &kallsyms, force_kallsyms))
 		goto out;
 	/**** No symbols or sections to be changed after kallsyms above ***/
 
@@ -1542,7 +1559,7 @@ int INSMOD_MAIN(int argc, char **argv)
 		goto out;
 
 	/* If we were just checking, we made it.  */
-	if (flag_silent_poll) {
+	if (flag_silent_probe) {
 		exit_status = 0;
 		goto out;
 	}
@@ -1591,7 +1608,7 @@ int INSMOD_MAIN(int argc, char **argv)
 		goto out;
 
 	/* Do kallsyms again, this time we have the final addresses */
-	if (add_kallsyms(f, &kallsyms))
+	if (add_kallsyms(f, &kallsyms, force_kallsyms))
 		goto out;
 
 #ifdef COMPAT_2_0
@@ -1674,14 +1691,14 @@ int main(int argc, char **argv)
 
 	for (i = 0; i < MAINS_NO; ++i) {
 		if (i) {
-			strcat(error_id1, "/");
+			xstrcat(error_id1, "/", sizeof(error_id1));
 			if (i == MAINS_NO-1)
-				strcat(error_id2, " or ");
+				xstrcat(error_id2, " or ", sizeof(error_id2));
 			else
-				strcat(error_id2, ", ");
+				xstrcat(error_id2, ", ", sizeof(error_id2));
 		}
-		strcat(error_id1, mains[i].name);
-		strcat(error_id2, mains[i].name);
+		xstrcat(error_id1, mains[i].name, sizeof(error_id1));
+		xstrcat(error_id2, mains[i].name, sizeof(error_id2));
 		if (strstr(p, mains[i].name)) {
 			++mains_match;
 			mains_which = i;
@@ -1690,8 +1707,8 @@ int main(int argc, char **argv)
 
 	/* Finish the error identifiers */
 	if (MAINS_NO != 1)
-		strcat(error_id1, " combined");
-	strcat(error_id1, " binary");
+		xstrcat(error_id1, " combined", sizeof(error_id1));
+	xstrcat(error_id1, " binary", sizeof(error_id1));
 
 	if (mains_match == 0 && MAINS_NO == 1)
 		++mains_match;		/* Not combined, any name will do */
