@@ -31,7 +31,7 @@
  *   Keith Owens <kaos@ocs.com.au> December 1999.
  */
 
-#ident "$Id: modinfo.c 1.11 Sun, 13 Aug 2000 23:45:56 +1000 kaos $"
+#ident "$Id: modinfo.c 1.11.1.2 Tue, 14 Nov 2000 23:36:27 +1100 kaos $"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -263,6 +263,10 @@ static void show_parameter(struct obj_file *f, char *key, char *value,
 	switch (*p) {
 	case 'c':
 		printf("char");
+		if (!isdigit(p[1]))
+			printf(" *** missing string size ***");
+		else while (isdigit(p[1]))
+			++p;	/* swallow c array size */
 		break;
 	case 'b':
 		printf("byte");
@@ -283,7 +287,7 @@ static void show_parameter(struct obj_file *f, char *key, char *value,
 		printf("no format character!\n");
 		return;
 	default:
-		printf("unknown format character %c", *p);
+		printf("unknown format character '%c'", *p);
 		return;
 	}
 
@@ -322,7 +326,7 @@ static void show_module_parameters(struct obj_file *f)
 				strncpy(pname, ptr + 5, namelen - 5);
 				pname[namelen - 5] = '\0';
 
-				strcpy(desckey, "parm_desc_");
+				strcpy(desckey, "parm_desc_");	/* safe, xmalloc */
 				strncat(desckey, ptr + 5, namelen - 5);
 				desckey[namelen + 5] = '\0';
 
