@@ -19,7 +19,7 @@
    along with this program; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
-#ident "$Id: obj_s390.c 1.3 Wed, 23 Aug 2000 10:23:26 +1000 kaos $"
+#ident "$Id: obj_s390.c 1.4 Wed, 15 Nov 2000 20:47:46 +1100 kaos $"
 
 #include <string.h>
 #include <assert.h>
@@ -227,6 +227,17 @@ arch_init_module (struct obj_file *f, struct module *m)
   return 1;
 }
 
+int         
+arch_finalize_section_address(struct obj_file *f, Elf32_Addr base)
+{    
+  int  i, n = f->header.e_shnum;
+             
+  f->baseaddr = base;
+  for (i = 0; i < n; ++i)
+    f->sections[i]->header.sh_addr += base;
+  return 1;  
+}
+
 int
 arch_archdata (struct obj_file *fin, struct obj_section *sec)
 {
