@@ -745,7 +745,7 @@ static int do_read(int all, char *force_ver, char *base_dir, char *conf_file, in
 				char **type;
 
 				for (type = tbtype; *type; ++type) {
-					char path[100];
+					char path[PATH_MAX];
 
 					snprintf(path, sizeof(path), "%s%s/%s", base_dir, *pathp, *type);
 					if (meta_expand(path, &g, NULL, version, ME_ALL))
