@@ -20,6 +20,8 @@
 #ident "$Id"
 
 #include <stdlib.h>
+#include <string.h>
+#include <memory.h>
 #include "util.h"
 
 
